@@ -5,7 +5,7 @@ from copy import deepcopy
 
 from .change_range import ChangeRangesBase, \
     Conv1DChangeRanges, \
-    MaxPooling1DChangeRanges, \
+    Pooling1DChangeRanges, \
     Cropping1DChangeRanges, \
     not_supported_error
 
@@ -39,9 +39,12 @@ AGGREGATE_LAYERS = {
 # layers at which output at ith position depends on a window around the ith position
 LOCAL_LAYERS = {
     'Conv1D',
-    'MaxPooling1D',
     'Cropping1D'
 }
+POOLING_LAYERS = {
+    'MaxPooling1D',
+    'AveragePooling1D'
+}
 
 # layers after which output at ith position depends on inputs at most or all positions
 # however, this is not true for Flatten/Reshape, but it is assumed these are followed
@@ -218,9 +221,9 @@ def segment_subgraph(current_node, nodes, edges, inbound_edges,
             # recursively label all descendants (no more further segments)
             return label_stop_descendants(current_node, nodes, edges, node_to_segment, segment_idx), stop_segment_idxs, segment_idx+1
 
-        elif (layer_class == 'MaxPooling1D' or layer_class == 'Cropping1D') \
+        elif (layer_class in POOLING_LAYERS or layer_class == 'Cropping1D') \
                 and segment_idx == 0:
-            # special case for when a Cropping or MaxPooling1D layer is right
+            # special case for when a Cropping or Pooling1D layer is right
             # after input sequence before first Conv1D
             segment_idx += 1
             node_to_segment[current_node] = segment_idx
@@ -235,7 +238,14 @@ def segment_subgraph(current_node, nodes, edges, inbound_edges,
             # enforce that if a segment has a conv layer, it is always at the beginning
             # by doing this, pre-conv intermediate outputs will always be captured and
             # padding them would become a one-time operation
-            segment_idx += 1
+
+            # however, if a segment has no previous convs (e.g. only SEE_THROUGH_LAYERS) 
+            # and has valid padding, then don't need to start a new segment 
+            # segment_idx==0 is special case for right after input sequence
+            # This corner case was discovered when trying to optimize for the Enformer
+            # model, which has blocks with batchnorm -> GELU -> width 1 Convs
+            if num_convs_in_cur_segment > 0 or segment_idx==0 or nodes[current_node].get_config()['padding'] != 'valid':
+                segment_idx += 1
 
             node_to_segment[current_node] = segment_idx
             return segment_subgraph(edges[current_node][0], nodes, edges, inbound_edges, node_to_segment, stop_segment_idxs, segment_idx, 1)
@@ -275,13 +285,15 @@ def segment_subgraph(current_node, nodes, edges, inbound_edges,
             return node_to_segment, stop_segment_idxs, segment_idx
 
 
-def label_stop_descendants(current_node, nodes, edges, node_to_segment, segment_idx):
-    # label nodes downstream of STOP_LAYERS
-    node_to_segment[current_node] = segment_idx
 
-    for node in edges[current_node]:
-        node_to_segment = label_stop_descendants(
-            node, nodes, edges, node_to_segment, segment_idx)
+def label_stop_descendants(current_node, nodes, edges, node_to_segment, segment_idx):
+    if current_node not in node_to_segment or node_to_segment[current_node] != segment_idx:
+        # label nodes downstream of STOP_LAYERS
+        node_to_segment[current_node] = segment_idx
+
+        for node in edges[current_node]:
+            node_to_segment = label_stop_descendants(
+                node, nodes, edges, node_to_segment, segment_idx)
 
     return node_to_segment
 
@@ -444,7 +456,7 @@ def compute_segment_change_ranges(model, nodes, edges, inbound_edges,
                     if flatten_model.node_is_layer(cur_segment_tensor):
                         layer_name = nodes[cur_segment_tensor].__class__.__name__
 
-                        if layer_name in LOCAL_LAYERS:
+                        if layer_name in LOCAL_LAYERS.union(POOLING_LAYERS):
                             if layer_name == 'Conv1D':
                                 change_range_objects.append(Conv1DChangeRanges(
                                     nodes[cur_segment_tensor].get_config()))
@@ -453,14 +465,14 @@ def compute_segment_change_ranges(model, nodes, edges, inbound_edges,
                                 segment_filters = nodes[cur_segment_tensor].get_config()[
                                     'filters']
 
-                            elif layer_name == 'MaxPooling1D':
-                                change_range_objects.append(MaxPooling1DChangeRanges(
-                                    nodes[cur_segment_tensor].get_config()))
-
                             elif layer_name == 'Cropping1D':
                                 change_range_objects.append(Cropping1DChangeRanges(
                                     nodes[cur_segment_tensor].get_config()))
 
+                            elif layer_name in POOLING_LAYERS:
+                                change_range_objects.append(Pooling1DChangeRanges(
+                                    nodes[cur_segment_tensor].get_config()))
+
                         elif (layer_name not in SEE_THROUGH_LAYERS) and \
                                 (layer_name not in AGGREGATE_LAYERS):
                             raise not_supported_error(
@@ -630,14 +642,18 @@ def generate_intermediate_output_subgraph(current_node, node_to_tensor,
                                                       stop_segment_idxs)
 
         # make the layer
-        layer = nodes[parent_layer].__class__(**config)
+        layer = nodes[parent_layer].__class__.from_config(config)
 
         if len(inbound_edges[parent_layer]) == 1:
             node_to_tensor[current_node] = layer(
                 node_to_tensor[inbound_edges[parent_layer][0]])
         else:
-            node_to_tensor[current_node] = layer(
-                [node_to_tensor[n] for n in inbound_edges[parent_layer]])
+            inps = [node_to_tensor[n] for n in inbound_edges[parent_layer]]
+
+            if layer.__class__.__name__ == 'MultiHeadAttention': 
+                node_to_tensor[current_node] = layer(*inps)
+            else:
+                node_to_tensor[current_node] = layer(inps)
 
         # set weights
         layer.set_weights(nodes[parent_layer].get_weights())
@@ -784,7 +800,7 @@ def generate_fast_ism_subgraph(current_node, node_edge_to_tensor, input_tensors,
                 if node_to_segment[parent_layer] not in stop_segment_idxs:
                     config['padding'] = 'valid'
 
-                layer = nodes[parent_layer].__class__(**config)
+                layer = nodes[parent_layer].__class__.from_config(config)
 
             elif layer_name == 'Flatten':
                 # this is necessary as SliceAssign loses dimension data
@@ -793,15 +809,19 @@ def generate_fast_ism_subgraph(current_node, node_edge_to_tensor, input_tensors,
                 layer = tf.keras.layers.Reshape(
                     nodes[parent_layer].output_shape[1:])
             else:
-                layer = nodes[parent_layer].__class__(**config)
+                layer = nodes[parent_layer].__class__.from_config(config)
 
             # call layer
             if len(inbound_edges[parent_layer]) == 1:
                 node_edge_to_tensor[(parent_layer, current_node)] = layer(
                     node_edge_to_tensor[(inbound_edges[parent_layer][0], parent_layer)])
             else:
-                node_edge_to_tensor[(parent_layer, current_node)] = layer(
-                    [node_edge_to_tensor[(n, parent_layer)] for n in inbound_edges[parent_layer]])
+                inps = [node_edge_to_tensor[(n, parent_layer)] for n in inbound_edges[parent_layer]]
+                
+                if layer_name == 'MultiHeadAttention':
+                    node_edge_to_tensor[(parent_layer, current_node)] = layer(*inps)
+                else:
+                    node_edge_to_tensor[(parent_layer, current_node)] = layer(inps)
 
             # set weights
             layer.set_weights(nodes[parent_layer].get_weights())
@@ -809,7 +829,7 @@ def generate_fast_ism_subgraph(current_node, node_edge_to_tensor, input_tensors,
     # if output edges of node have different segment
     # and both segments are not stop segments
     # perform slice assignment and store relevant tensor
-    for next_layer_node in edges[current_node]:
+    for next_layer_node in set(edges[current_node]):
         if (node_to_segment[current_node] != node_to_segment[next_layer_node]) and \
             ((node_to_segment[current_node] not in stop_segment_idxs) or
                 (node_to_segment[next_layer_node] not in stop_segment_idxs)):
